import requests
import datetime
from concurrent.futures import ThreadPoolExecutor, as_completed

# Define constants
COLLECT_CATALOGUS = ('https://waterwebservices.rijkswaterstaat.nl/ONLINEWAARNEMINGENSERVICES_DBO/OphalenWaarnemingen')
LOCATIES = [
             {"Code": "VLIS", "X": 541518.745919649, "Y": 5699254.96425966 }
             ,{"Code": "EMSH", "X": 754251.664251634, "Y": 5928910.1372502 }
             ,{"Code": "IJMDBNN", "X": 608822.598508896, "Y": 5814553.68258518 }
             ,{"Code":"IJMDNDSS", "X": 609033.673550824, "Y": 5814220.53361595 }  
             ,{"Code":"IJMDSMPL", "X": 603100.219284376, "Y": 5813724.18174203 } 
             ,{"Code":"IJMDSMFR", "X": 606962.047312289, "Y": 5813641.93165918}
             ,{"Code":"IJMDZDLHVHD", "X": 604098.569701313, "Y": 5813717.41966292}           
             ,{"Code":"IJMH", "X": 605632.956827924, "Y": 5813598.08945097 }
             ,{"Code":"IJMO", "X": 609559.811682633, "Y": 5814278.94993479 }
             ,{"Code":"IJMW", "X": 609009.433796157, "Y": 5814225.74243423 }
             ,{"Code":"ROTTDM", "X": 603140.026100252, "Y": 5753205.34858951 }
             ,{"Code":"VLISSGN", "X": 541425.983214885, "Y": 5699181.90968435 }
             ,{"Code":"VLAA", "X": 592852.349051705, "Y": 5750773.29674651 }
             ,{"Code":"HOEKVHLD", "X": 576917.669784491, "Y": 5759136.15818497 }
             ,{"Code":"HOEKVHLDX", "X": 776917.669784491, "Y": 1759136.15818497 }
            ]

START_YEAR = 2014
END_YEAR = 2024
NUM_YEARS = END_YEAR - START_YEAR + 1

# Clear the results and errors files before starting
def clear_files():
    with open("results.csv", "w"), open("errors.log", "w"), open("response.json", "w"):
        pass  # This clears the files

# Function to create a request payload
def create_request_payload(locatie, start_date, end_date):
    return {
        "Locatie": locatie,
        "AquoPlusWaarnemingMetadata": {
            "AquoMetadata": {"Compartiment": {"Code": "OW"}, "Grootheid": {"Code": "WATHTE"}},
            "WaarnemingMetadata": {"KwaliteitswaardecodeLijst": ["00", "10", "20", "25", "30", "40"]}
        },
        "Periode": {
            "Begindatumtijd": f"{start_date}T08:00:00.000+01:00",
            "Einddatumtijd": f"{end_date}T23:00:00.000+01:00"
        }
    }

# Function to process a single request
def process_request(locatie, start_date, end_date):
    request_payload = create_request_payload(locatie, start_date, end_date)
    try:
        response = requests.post(COLLECT_CATALOGUS, json=request_payload)
        response.raise_for_status()  # Raise an error for bad status codes
        elements = response.json()
        #log_response.write(f' {resp.status_code}; {elements}')

        ## It is beter if reponse not give 200 by finding no data  
        if response.status_code == 200:
            ## This is a check for Succesvol is false. It is beter if reponse not give 200 by finding no data 
             if 'Foutmelding' in elements:
                 log_error(f"Request failed for locatie {locatie} from {start_date} to {end_date}: Status Code {response.status_code} Errror: {elements['Foutmelding']}")                       
             else:
                with open("results.csv", "a") as result_file:
                    for waarnemingen in elements.get('WaarnemingenLijst', []):
                        for lijst in waarnemingen.get('MetingenLijst', []):
                            result = (locatie['Code'], locatie['X'], locatie['Y'], lijst['Meetwaarde']['Waarde_Numeriek'], lijst['Tijdstip'])
                            result_file.write(f"{result};\n")


    except requests.RequestException as e:
        log_error(f"Request failed for locatie {locatie} from {start_date} to {end_date}: {e}")
    except KeyError as e:
        log_error(f"Unexpected response format for locatie {locatie} from {start_date} to {end_date}: missing key {e}")

# Function to log errors
def log_error(message):
    with open("errors.log", "a") as error_file:
        error_file.write(message + "\n")

# Loop through each year and each locatie
def generate_tasks():
    tasks = []
    for year_offset in range(NUM_YEARS):
        start_date = f"{START_YEAR + year_offset}-01-01"
        end_date = f"{START_YEAR + year_offset}-12-31"

        # Adjust end date for the year 2024
        if START_YEAR + year_offset == 2024:
            end_date = "2024-08-31"
        
        tasks.extend((locatie, start_date, end_date) for locatie in LOCATIES)
    return tasks

# Main function to handle parallel processing of requests
def main():
    clear_files()
    tasks = generate_tasks()
    with ThreadPoolExecutor(max_workers=8) as executor:
        future_to_task = {executor.submit(process_request, locatie, start_date, end_date): (locatie, start_date, end_date) for locatie, start_date, end_date in tasks}
        
        for future in as_completed(future_to_task):
            locatie, start_date, end_date = future_to_task[future]
            try:
                future.result()  # If an exception occurred, it will be raised here
            except Exception as e:
                log_error(f"Error processing task for {locatie} from {start_date} to {end_date}: {e}")

if __name__ == "__main__":
    main()
